from PyCode import PyStat, PyStats, PySuiteStat

def p_declarations(s):
  # Parse a sequence of declarations, return Python code for it.
  code = PyStats()
  while s.sy:
    if s.sy == '<':
      s.next()
      if s.atom('ROUTINE'):
        s.next()
        code.append(p_routine(s))
      else:
        s.error("%s declaration not yet supported" % s.systring)
      s.expect('>')
    else:
      s.expected('<')
  return code

def p_routine(s):
  name = p_atom(s)
  required_args = []
  optional_args = []
  local_vars = []
  s.expect('(')
  p_atoms(s, required_args)
  if s.keyword("OPT"):
    s.next()
    p_atoms(s, optional_args)
  if s.keyword("AUX"):
    s.next()
    p_atoms(s, local_vars)
  s.expect(')')
  required_arg_code = [local_var_name(atom) for atom in required_args]
  optional_arg_code = ["%s=0" % local_var_name(atom) for atom in optional_args]
  arg_code = ", ".join(required_arg_code + optional_arg_code)
  header = "def %s(%s)" % (routine_name(name), arg_code)
  body = p_operations(s)
  return PySuiteStat(header, body)

def p_operations(s):
  code = PyStats()
  while s.sy == '<':
    code.append(PyStat(p_operation(s)))
  return code

def binop_format(op):
  return (2, "(%%s %s %%s)" % op)

builtin_operations = {
  'ADD':   binop_format("+"),
  '+':     binop_format("+"),
  'SUB':   binop_format("-"),
  '-':     binop_format("-"),
  'MUL':   binop_format("*"),
  '*':     binop_format("*"),
  'DIV':   binop_format("/"),
  '/':     binop_format("/"),
  'MOD':   binop_format("%"),
  'LESS?': binop_format("<"),
  'L?':    binop_format("<"),
  'GRTR?': binop_format(">"),
  'G?':    binop_format(">"),
  'AND':   binop_format("and"),
  'OR':    binop_format("or"),
  'NOT':   binop_format("not"),
}
  
def p_operation(s):
  s.expect('<')
  operands = []
  if s.sy == 'ATOM' and builtin_operations.has_key(s.systring):
    nargs, format = builtin_operations[s.systring]
    s.next()
    for i in range(nargs):
      operands.append(p_expr(s))
    result = format % tuple(operands)
  elif s.atom('SETG', 'SET'):
    op = s.systring
    s.next()
    name = p_atom(s)
    value = p_expr(s)
    if op == 'SETG':
      name_code = global_var_name(name)
    else:
      name_code = local_var_name(name)
    result = "%s = %s" % (name_code, value)
  else:
    routine = p_expr(s)
    while s.sy and s.sy <> '>':
      operands.append(p_expr(s))
    result = "%s(%s)" % (routine, ",".join(operands))
  s.expect('>')
  return result

def p_expr(s):
  sy = s.sy
  if sy == '<':
    result = p_operation(s)
  else:
    if sy == 'INTEGER':
      result = s.systring
      s.next()
    elif sy == 'STRING':
      result = '""%s""' % s.systring
      s.next()
    elif sy == '.':
      s.next()
      result = local_var_name(p_atom(s))
    elif sy == ',':
      s.next()
      result = global_var_name(p_atom(s))
    elif sy == 'ATOM':
      result = routine_name(s.systring)
      s.next()
    else:
      s.expected("an expression")
  return result

def p_atoms(s, result):
  while s.sy == 'ATOM':
    result.append(s.systring)
    s.next()

def p_atom(s):
  if s.sy == 'ATOM':
    result = s.systring
    s.next()
    return result
  else:
    s.expected("an atom")

def global_var_name(atom):
  return "g.%s" % atom

def local_var_name(atom):
  return "l_%s" % atom

def routine_name(atom):
  return "r_%s" % atom
