import sys
from Plex import *

punct = "()<>,."
space = " \t\n"
integer = Rep1(Range("09"))
string = Str('"') + Rep(AnyBut('"')) + Str('"')

class ZilError(Exception):
  pass

class ZilScanner(Scanner):

  def atom_action(self, text):
    self.produce('ATOM', text.upper())

  lexicon = Lexicon([
    (Any(punct), TEXT),
    (integer, 'INTEGER'),
    (string, 'STRING'),
    (Rep1(AnyBut(punct + space + '"')), atom_action),
    (Any(space), IGNORE),
    #(AnyChar, 'WHAT?'),
  ])

  def __init__(self, file, name):
    Scanner.__init__(self, self.lexicon, file, name)
    self.next()

  def next(self):
    self.sy, self.systring = self.read()

  def atom(self, *what):
    return self.sy == 'ATOM' and self.systring in what

  def keyword(self, what):
    return self.sy == 'STRING' and self.systring[1:-1].upper() == what

  def error(self, mess):
    filename, line, char = self.position()
    sys.stderr.write("%s:%s: %s\n" % (filename, line, mess))
    raise ZilError

  def expect(self, what):
    if self.sy == what:
      self.next()
    else:
      self.expected('%s' % what)

  def expected(self, what):
    self.error("Expected %s, but found %s" % (what, self.systring))
